@@ -33,7 +33,7 @@ $specialPageAliases['ba'] = [
 	'Nuke' => [ 'Nuke' ],
 ];
 
-/** Belarusian (Taraškievica orthography) (беларуская (тарашкевіца)‎) */
+/** Belarusian (Taraškievica orthography) (беларуская (тарашкевіца)) */
 $specialPageAliases['be-tarask'] = [
 	'Nuke' => [ 'Масавае_выдаленьне' ],
 ];
@@ -263,6 +263,16 @@ $specialPageAliases['sk'] = [
 	'Nuke' => [ 'Zničenie' ],
 ];
 
+/** Serbian (Cyrillic script) (српски (ћирилица)) */
+$specialPageAliases['sr-ec'] = [
+	'Nuke' => [ 'МасовноБрисање', 'Масовно_брисање' ],
+];
+
+/** Serbian (Latin script) (srpski (latinica)) */
+$specialPageAliases['sr-el'] = [
+	'Nuke' => [ 'MasovnoBrisanje', 'Masovno_brisanje' ],
+];
+
 /** Swedish (svenska) */
 $specialPageAliases['sv'] = [
 	'Nuke' => [ 'Massradering' ],
@@ -293,17 +303,12 @@ $specialPageAliases['vi'] = [
 	'Nuke' => [ 'Xóa_hàng_loạt', 'Xoá_hàng_loạt' ],
 ];
 
-/** Chinese (中文) */
-$specialPageAliases['zh'] = [
-	'Nuke' => [ '大量删除', '大量刪除' ],
-];
-
-/** Simplified Chinese (中文（简体）‎) */
+/** Simplified Chinese (中文（简体）) */
 $specialPageAliases['zh-hans'] = [
 	'Nuke' => [ '大量删除' ],
 ];
 
-/** Traditional Chinese (中文（繁體）‎) */
+/** Traditional Chinese (中文（繁體）) */
 $specialPageAliases['zh-hant'] = [
 	'Nuke' => [ '大量刪除' ],
 ];
