@@ -23,12 +23,44 @@
  * @author Rob Church <robchur@gmail.com>
  */
 
+use MediaWiki\Context\IContextSource;
+use MediaWiki\HTMLForm\HTMLForm;
+use MediaWiki\Language\Language;
+use MediaWiki\MediaWikiServices;
+use MediaWiki\Output\OutputPage;
+use MediaWiki\Status\Status;
+use MediaWiki\User\User;
+use MediaWiki\Utils\MWTimestamp;
+
 /**
  * File reversion user interface
+ * WikiPage must contain getFile method: \WikiFilePage
+ * Article::getFile is only for b/c: \ImagePage
  *
  * @ingroup Actions
  */
 class RevertAction extends FormAction {
+
+	private Language $contentLanguage;
+	private RepoGroup $repoGroup;
+
+	/**
+	 * @param Article $article
+	 * @param IContextSource $context
+	 * @param Language $contentLanguage
+	 * @param RepoGroup $repoGroup
+	 */
+	public function __construct(
+		Article $article,
+		IContextSource $context,
+		Language $contentLanguage,
+		RepoGroup $repoGroup
+	) {
+		parent::__construct( $article, $context );
+		$this->contentLanguage = $contentLanguage;
+		$this->repoGroup = $repoGroup;
+	}
+
 	/**
 	 * @var OldLocalFile
 	 */
@@ -39,6 +71,7 @@ class RevertAction extends FormAction {
 	}
 
 	public function getRestriction() {
+		// Required permissions of revert are complicated, will be checked below.
 		return 'upload';
 	}
 
@@ -46,6 +79,16 @@ class RevertAction extends FormAction {
 		if ( $this->getTitle()->getNamespace() !== NS_FILE ) {
 			throw new ErrorPageError( $this->msg( 'nosuchaction' ), $this->msg( 'nosuchactiontext' ) );
 		}
+
+		$rights = [ 'reupload' ];
+		if ( $user->equals( $this->getFile()->getUploader() ) ) {
+			// reupload-own is more basic, put it in the front for error messages.
+			array_unshift( $rights, 'reupload-own' );
+		}
+		if ( !$user->isAllowedAny( ...$rights ) ) {
+			throw new PermissionsError( $rights[0] );
+		}
+
 		parent::checkCanExecute( $user );
 
 		$oldimage = $this->getRequest()->getText( 'oldimage' );
@@ -56,10 +99,8 @@ class RevertAction extends FormAction {
 			throw new ErrorPageError( 'internalerror', 'unexpected', [ 'oldimage', $oldimage ] );
 		}
 
-		$this->oldFile = RepoGroup::singleton()->getLocalRepo()->newFromArchiveName(
-			$this->getTitle(),
-			$oldimage
-		);
+		$this->oldFile = $this->repoGroup->getLocalRepo()
+			->newFromArchiveName( $this->getTitle(), $oldimage );
 
 		if ( !$this->oldFile->exists() ) {
 			throw new ErrorPageError( '', 'filerevert-badversion' );
@@ -78,8 +119,6 @@ class RevertAction extends FormAction {
 	}
 
 	protected function getFormFields() {
-		global $wgContLang;
-
 		$timestamp = $this->oldFile->getTimestamp();
 
 		$user = $this->getUser();
@@ -88,19 +127,22 @@ class RevertAction extends FormAction {
 		$userTime = $lang->userTime( $timestamp, $user );
 		$siteTs = MWTimestamp::getLocalInstance( $timestamp );
 		$ts = $siteTs->format( 'YmdHis' );
-		$siteDate = $wgContLang->date( $ts, false, false );
-		$siteTime = $wgContLang->time( $ts, false, false );
+		$contLang = $this->contentLanguage;
+		$siteDate = $contLang->date( $ts, false, false );
+		$siteTime = $contLang->time( $ts, false, false );
 		$tzMsg = $siteTs->getTimezoneMessage()->inContentLanguage()->text();
 
 		return [
 			'intro' => [
 				'type' => 'info',
-				'vertical-label' => true,
 				'raw' => true,
 				'default' => $this->msg( 'filerevert-intro',
 					$this->getTitle()->getText(), $userDate, $userTime,
-					wfExpandUrl(
-						$this->page->getFile()->getArchiveUrl( $this->getRequest()->getText( 'oldimage' ) ),
+					(string)MediaWikiServices::getInstance()->getUrlUtils()->expand(
+						$this->getFile()
+							->getArchiveUrl(
+								$this->getRequest()->getText( 'oldimage' )
+							),
 						PROTO_CURRENT
 					) )->parseAsBlock()
 			],
@@ -117,7 +159,9 @@ class RevertAction extends FormAction {
 		$this->useTransactionalTimeLimit();
 
 		$old = $this->getRequest()->getText( 'oldimage' );
-		$localFile = $this->page->getFile();
+		/** @var LocalFile $localFile */
+		$localFile = $this->getFile();
+		'@phan-var LocalFile $localFile';
 		$oldFile = OldLocalFile::newFromArchiveName( $this->getTitle(), $localFile->getRepo(), $old );
 
 		$source = $localFile->getArchiveVirtualUrl( $old );
@@ -135,7 +179,10 @@ class RevertAction extends FormAction {
 			0,
 			false,
 			false,
-			$this->getUser()
+			$this->getAuthority(),
+			[],
+			true,
+			true
 		);
 	}
 
@@ -148,21 +195,36 @@ class RevertAction extends FormAction {
 
 		$this->getOutput()->addWikiMsg( 'filerevert-success', $this->getTitle()->getText(),
 			$userDate, $userTime,
-			wfExpandUrl( $this->page->getFile()->getArchiveUrl( $this->getRequest()->getText( 'oldimage' ) ),
+			(string)MediaWikiServices::getInstance()->getUrlUtils()->expand(
+				$this->getFile()
+					->getArchiveUrl(
+						$this->getRequest()->getText( 'oldimage' )
+					),
 				PROTO_CURRENT
-		) );
+			) );
 		$this->getOutput()->returnToMain( false, $this->getTitle() );
 	}
 
 	protected function getPageTitle() {
-		return $this->msg( 'filerevert', $this->getTitle()->getText() );
+		return $this->msg( 'filerevert' )->plaintextParams( $this->getTitle()->getText() );
 	}
 
 	protected function getDescription() {
-		return OutputPage::buildBacklinkSubtitle( $this->getTitle() );
+		return OutputPage::buildBacklinkSubtitle( $this->getTitle() )->escaped();
 	}
 
 	public function doesWrites() {
 		return true;
 	}
+
+	/**
+	 * @since 1.35
+	 * @return File
+	 */
+	private function getFile(): File {
+		/** @var \WikiFilePage $wikiPage */
+		$wikiPage = $this->getWikiPage();
+		// @phan-suppress-next-line PhanUndeclaredMethod
+		return $wikiPage->getFile();
+	}
 }
