@@ -18,11 +18,15 @@
  * @file
  */
 
+use MediaWiki\Language\Language;
+use MediaWiki\Languages\LanguageFactory;
+use MediaWiki\MediaWikiServices;
+
 /**
  * @since 1.16.3
  */
 class IcuCollation extends Collation {
-	const FIRST_LETTER_VERSION = 3;
+	private const FIRST_LETTER_VERSION = 4;
 
 	/** @var Collator */
 	private $primaryCollator;
@@ -51,7 +55,7 @@ class IcuCollation extends Collation {
 	 * is pretty useless for sorting Chinese text anyway. Japanese and Korean
 	 * blocks are not included here, because they are smaller and more useful.
 	 */
-	private static $cjkBlocks = [
+	private const CJK_BLOCKS = [
 		[ 0x2E80, 0x2EFF ], // CJK Radicals Supplement
 		[ 0x2F00, 0x2FDF ], // Kangxi Radicals
 		[ 0x2FF0, 0x2FFF ], // Ideographic Description Characters
@@ -75,8 +79,8 @@ class IcuCollation extends Collation {
 	 * letters (denoted by keys starting with '-').
 	 *
 	 * These are additions to (or subtractions from) the data stored in the
-	 * first-letters-root.ser file (which among others includes full basic latin,
-	 * cyrillic and greek alphabets).
+	 * first-letters-root.php data file (which among others includes full basic
+	 * Latin, Cyrillic and Greek alphabets).
 	 *
 	 * "Separate letter" is a letter that would have a separate heading/section
 	 * for it in a dictionary or a phone book in this language. This data isn't
@@ -90,11 +94,11 @@ class IcuCollation extends Collation {
 	 * Empty arrays are intended; this signifies that the data for the language is
 	 * available and that there are, in fact, no additional letters to consider.
 	 */
-	private static $tailoringFirstLetters = [
+	private const TAILORING_FIRST_LETTERS = [
 		'af' => [],
 		'am' => [],
 		'ar' => [],
-		'as' => [ "\xe0\xa6\x82", "\xe0\xa6\x81", "\xe0\xa6\x83", "\xe0\xa7\x8e", "ক্ষ " ],
+		'as' => [ "\u{0982}", "\u{0981}", "\u{0983}", "\u{09CE}", "ক্ষ " ],
 		'ast' => [ "Ch", "Ll", "Ñ" ], // not in libicu
 		'az' => [ "Ç", "Ə", "Ğ", "İ", "Ö", "Ş", "Ü" ],
 		'be' => [ "Ё" ],
@@ -124,7 +128,7 @@ class IcuCollation extends Collation {
 		'en' => [],
 		'eo' => [ "Ĉ", "Ĝ", "Ĥ", "Ĵ", "Ŝ", "Ŭ" ],
 		'es' => [ "Ñ" ],
-		'et' => [ "Š", "Ž", "Õ", "Ä", "Ö", "Ü", "W" ], // added W for CollationEt (xx-uca-et)
+		'et' => [ "Š", "Ž", "Õ", "Ä", "Ö", "Ü" ],
 		'eu' => [ "Ñ" ], // not in libicu
 		'fa' => [
 			// RTL, let's put each letter on a new line
@@ -144,11 +148,11 @@ class IcuCollation extends Collation {
 		'ga' => [],
 		'gd' => [], // not in libicu
 		'gl' => [ "Ch", "Ll", "Ñ" ],
-		'gu' => [ "\xe0\xaa\x82", "\xe0\xaa\x83", "\xe0\xaa\x81", "\xe0\xaa\xb3" ],
+		'gu' => [ "\u{0A82}", "\u{0A83}", "\u{0A81}", "\u{0AB3}" ],
 		'ha' => [ 'Ɓ', 'Ɗ', 'Ƙ', 'Sh', 'Ts', 'Ƴ' ],
 		'haw' => [ 'ʻ' ],
 		'he' => [],
-		'hi' => [ "\xe0\xa4\x82", "\xe0\xa4\x83" ],
+		'hi' => [ "\u{0902}", "\u{0903}" ],
 		'hr' => [ "Č", "Ć", "Dž", "Đ", "Lj", "Nj", "Š", "Ž" ],
 		'hsb' => [ "Č", "Dź", "Ě", "Ch", "Ł", "Ń", "Ř", "Š", "Ć", "Ž" ],
 		'hu' => [ "Cs", "Dz", "Dzs", "Gy", "Ly", "Ny", "Ö", "Sz", "Ty", "Ü", "Zs" ],
@@ -161,13 +165,13 @@ class IcuCollation extends Collation {
 		'kk' => [ "Ү", "І" ],
 		'kl' => [ "Æ", "Ø", "Å" ],
 		'km' => [
-			"រ", "ឫ", "ឬ", "ល", "ឭ", "ឮ", "\xe1\x9e\xbb\xe1\x9f\x86",
-			"\xe1\x9f\x86", "\xe1\x9e\xb6\xe1\x9f\x86", "\xe1\x9f\x87",
-			"\xe1\x9e\xb7\xe1\x9f\x87", "\xe1\x9e\xbb\xe1\x9f\x87",
-			"\xe1\x9f\x81\xe1\x9f\x87", "\xe1\x9f\x84\xe1\x9f\x87",
+			"រ", "ឫ", "ឬ", "ល", "ឭ", "ឮ", "\u{17BB}\u{17C6}",
+			"\u{17C6}", "\u{17B6}\u{17C6}", "\u{17C7}",
+			"\u{17B7}\u{17C7}", "\u{17BB}\u{17C7}",
+			"\u{17C1}\u{17C7}", "\u{17C4}\u{17C7}",
 		],
-		'kn' => [ "\xe0\xb2\x81", "\xe0\xb2\x83", "\xe0\xb3\xb1", "\xe0\xb3\xb2" ],
-		'kok' => [ "\xe0\xa4\x82", "\xe0\xa4\x83", "ळ", "क्ष" ],
+		'kn' => [ "\u{0C81}", "\u{0C83}", "\u{0CF1}", "\u{0CF2}" ],
+		'kok' => [ "\u{0902}", "\u{0903}", "ळ", "क्ष" ],
 		'ku' => [ "Ç", "Ê", "Î", "Ş", "Û" ], // not in libicu
 		'ky' => [ "Ё" ],
 		'la' => [], // not in libicu
@@ -181,7 +185,7 @@ class IcuCollation extends Collation {
 		'ml' => [],
 		'mn' => [],
 		'mo' => [ "Ă", "Â", "Î", "Ș", "Ț" ], // not in libicu
-		'mr' => [ "\xe0\xa4\x82", "\xe0\xa4\x83", "ळ", "क्ष", "ज्ञ" ],
+		'mr' => [ "\u{0902}", "\u{0903}", "ळ", "क्ष", "ज्ञ" ],
 		'ms' => [],
 		'mt' => [ "Ċ", "Ġ", "Għ", "Ħ", "Ż" ],
 		'nb' => [ "Æ", "Ø", "Å" ],
@@ -191,8 +195,8 @@ class IcuCollation extends Collation {
 		'no' => [ "Æ", "Ø", "Å" ], // not in libicu. You should probably use nb or nn instead.
 		'oc' => [], // not in libicu
 		'om' => [ 'Ch', 'Dh', 'Kh', 'Ny', 'Ph', 'Sh' ],
-		'or' => [ "\xe0\xac\x81", "\xe0\xac\x82", "\xe0\xac\x83", "କ୍ଷ" ],
-		'pa' => [ "\xe0\xa9\x8d" ],
+		'or' => [ "\u{0B01}", "\u{0B02}", "\u{0B03}", "କ୍ଷ" ],
+		'pa' => [ "\u{0A4D}" ],
 		'pl' => [ "Ą", "Ć", "Ę", "Ł", "Ń", "Ó", "Ś", "Ź", "Ż" ],
 		'pt' => [],
 		'rm' => [], // not in libicu
@@ -204,7 +208,7 @@ class IcuCollation extends Collation {
 			'Á', 'Č', 'Ʒ', 'Ǯ', 'Đ', 'Ǧ', 'Ǥ', 'Ǩ', 'Ŋ',
 			'Š', 'Ŧ', 'Ž', 'Ø', 'Æ', 'Ȧ', 'Ä', 'Ö'
 		],
-		'si' => [ "\xe0\xb6\x82", "\xe0\xb6\x83", "\xe0\xb6\xa4" ],
+		'si' => [ "\u{0D82}", "\u{0D83}", "\u{0DA4}" ],
 		'sk' => [ "Ä", "Č", "Ch", "Ô", "Š", "Ž" ],
 		'sl' => [ "Č", "Š", "Ž" ],
 		'smn' => [ "Á", "Č", "Đ", "Ŋ", "Š", "Ŧ", "Ž", "Æ", "Ø", "Å", "Ä", "Ö" ],
@@ -215,55 +219,55 @@ class IcuCollation extends Collation {
 		'sv@collation=standard' => [ "Å", "Ä", "Ö" ],
 		'sw' => [],
 		'ta' => [
-			"\xE0\xAE\x82", "ஃ", "க்ஷ", "க்", "ங்", "ச்", "ஞ்", "ட்", "ண்", "த்", "ந்",
+			"\u{0B82}", "ஃ", "க்ஷ", "க்", "ங்", "ச்", "ஞ்", "ட்", "ண்", "த்", "ந்",
 			"ப்", "ம்", "ய்", "ர்", "ல்", "வ்", "ழ்", "ள்", "ற்", "ன்", "ஜ்", "ஶ்", "ஷ்",
 			"ஸ்", "ஹ்", "க்ஷ்"
 		],
-		'te' => [ "\xe0\xb0\x81", "\xe0\xb0\x82", "\xe0\xb0\x83" ],
-		'th' => [ "ฯ", "\xe0\xb9\x86", "\xe0\xb9\x8d", "\xe0\xb8\xba" ],
+		'te' => [ "\u{0C01}", "\u{0C02}", "\u{0C03}" ],
+		'th' => [ "ฯ", "\u{0E46}", "\u{0E4D}", "\u{0E3A}" ],
 		'tk' => [ "Ç", "Ä", "Ž", "Ň", "Ö", "Ş", "Ü", "Ý" ],
 		'tl' => [ "Ñ", "Ng" ], // not in libicu
 		'to' => [ "Ng", "ʻ" ],
 		'tr' => [ "Ç", "Ğ", "İ", "Ö", "Ş", "Ü" ],
+		'-tr' => [ "ı" ],
 		'tt' => [ "Ә", "Ө", "Ү", "Җ", "Ң", "Һ" ], // not in libicu
 		'uk' => [ "Ґ", "Ь" ],
 		'uz' => [ "Ch", "G'", "Ng", "O'", "Sh" ], // not in libicu
 		'vi' => [ "Ă", "Â", "Đ", "Ê", "Ô", "Ơ", "Ư" ],
 		'vo' => [ "Ä", "Ö", "Ü" ],
 		'yi' => [
-			"\xd7\x91\xd6\xbf", "\xd7\x9b\xd6\xbc", "\xd7\xa4\xd6\xbc",
-			"\xd7\xa9\xd7\x82", "\xd7\xaa\xd6\xbc"
+			"\u{05D1}\u{05BF}", "\u{05DB}\u{05BC}", "\u{05E4}\u{05BC}",
+			"\u{05E9}\u{05C2}", "\u{05EA}\u{05BC}"
 		],
 		'yo' => [ "Ẹ", "Gb", "Ọ", "Ṣ" ],
 		'zu' => [],
 	];
 
 	/**
-	 * @since 1.16.3
+	 * @param LanguageFactory $languageFactory
+	 * @param string $locale
 	 */
-	const RECORD_LENGTH = 14;
-
-	public function __construct( $locale ) {
-		if ( !extension_loaded( 'intl' ) ) {
-			throw new MWException( 'An ICU collation was requested, ' .
-				'but the intl extension is not available.' );
-		}
-
+	public function __construct(
+		LanguageFactory $languageFactory,
+		$locale
+	) {
 		$this->locale = $locale;
 		// Drop everything after the '@' in locale's name
 		$localeParts = explode( '@', $locale );
-		$this->digitTransformLanguage = Language::factory( $locale === 'root' ? 'en' : $localeParts[0] );
+		$this->digitTransformLanguage = $languageFactory->getLanguage( $locale === 'root' ? 'en' : $localeParts[0] );
 
-		$this->mainCollator = Collator::create( $locale );
-		if ( !$this->mainCollator ) {
-			throw new MWException( "Invalid ICU locale specified for collation: $locale" );
+		$mainCollator = Collator::create( $locale );
+		if ( !$mainCollator ) {
+			throw new InvalidArgumentException( "Invalid ICU locale specified for collation: $locale" );
 		}
+		$this->mainCollator = $mainCollator;
 
+		// @phan-suppress-next-line PhanPossiblyNullTypeMismatchProperty successed before, no null check needed
 		$this->primaryCollator = Collator::create( $locale );
 		$this->primaryCollator->setStrength( Collator::PRIMARY );
 
 		// If the special suffix for numeric collation is present, turn on numeric collation.
-		if ( substr( $locale, -5, 5 ) === '-u-kn' ) {
+		if ( str_ends_with( $locale, '-u-kn' ) ) {
 			$this->useNumericCollation = true;
 			// Strip off the special suffix so it doesn't trip up fetchFirstLetterData().
 			$this->locale = substr( $this->locale, 0, -5 );
@@ -276,10 +280,6 @@ class IcuCollation extends Collation {
 		return $this->mainCollator->getSortKey( $string );
 	}
 
-	public function getPrimarySortKey( $string ) {
-		return $this->primaryCollator->getSortKey( $string );
-	}
-
 	public function getFirstLetter( $string ) {
 		$string = strval( $string );
 		if ( $string === '' ) {
@@ -289,16 +289,21 @@ class IcuCollation extends Collation {
 		$firstChar = mb_substr( $string, 0, 1, 'UTF-8' );
 
 		// If the first character is a CJK character, just return that character.
-		if ( ord( $firstChar ) > 0x7f && self::isCjk( UtfNormal\Utils::utf8ToCodepoint( $firstChar ) ) ) {
+		if ( ord( $firstChar ) > 0x7f && self::isCjk( mb_ord( $firstChar ) ) ) {
 			return $firstChar;
 		}
 
 		$sortKey = $this->getPrimarySortKey( $string );
+		$data = $this->getFirstLetterData();
+		$keys = $data['keys'];
+		$letters = $data['chars'];
 
 		// Do a binary search to find the correct letter to sort under
 		$min = ArrayUtils::findLowerBound(
-			[ $this, 'getSortKeyByLetterIndex' ],
-			$this->getFirstLetterCount(),
+			static function ( $index ) use ( $keys ) {
+				return $keys[$index];
+			},
+			count( $keys ),
 			'strcmp',
 			$sortKey );
 
@@ -307,7 +312,7 @@ class IcuCollation extends Collation {
 			return '';
 		}
 
-		$sortLetter = $this->getLetterByIndex( $min );
+		$sortLetter = $letters[$min];
 
 		if ( $this->useNumericCollation ) {
 			// If the sort letter is a number, return '0–9' (or localized equivalent).
@@ -321,19 +326,24 @@ class IcuCollation extends Collation {
 		return $sortLetter;
 	}
 
+	private function getPrimarySortKey( $string ) {
+		return $this->primaryCollator->getSortKey( $string );
+	}
+
 	/**
 	 * @since 1.16.3
 	 * @return array
 	 */
-	public function getFirstLetterData() {
+	private function getFirstLetterData() {
 		if ( $this->firstLetterData === null ) {
-			$cache = ObjectCache::getLocalServerInstance( CACHE_ANYTHING );
+			$cache = MediaWikiServices::getInstance()->getObjectCacheFactory()
+				->getLocalServerInstance( CACHE_ANYTHING );
 			$cacheKey = $cache->makeKey(
 				'first-letters',
 				static::class,
 				$this->locale,
 				$this->digitTransformLanguage->getCode(),
-				self::getICUVersion(),
+				INTL_ICU_VERSION,
 				self::FIRST_LETTER_VERSION
 			);
 			$this->firstLetterData = $cache->getWithSetCallback( $cacheKey, $cache::TTL_WEEK, function () {
@@ -345,30 +355,28 @@ class IcuCollation extends Collation {
 
 	/**
 	 * @return array
-	 * @throws MWException
 	 */
 	private function fetchFirstLetterData() {
 		// Generate data from serialized data file
-		if ( isset( self::$tailoringFirstLetters[$this->locale] ) ) {
-			$letters = wfGetPrecompiledData( 'first-letters-root.ser' );
+		if ( isset( self::TAILORING_FIRST_LETTERS[$this->locale] ) ) {
+			$letters = require __DIR__ . "/data/first-letters-root.php";
 			// Append additional characters
-			$letters = array_merge( $letters, self::$tailoringFirstLetters[$this->locale] );
+			$letters = array_merge( $letters, self::TAILORING_FIRST_LETTERS[$this->locale] );
 			// Remove unnecessary ones, if any
-			if ( isset( self::$tailoringFirstLetters['-' . $this->locale] ) ) {
-				$letters = array_diff( $letters, self::$tailoringFirstLetters['-' . $this->locale] );
+			if ( isset( self::TAILORING_FIRST_LETTERS['-' . $this->locale] ) ) {
+				$letters = array_diff( $letters, self::TAILORING_FIRST_LETTERS['-' . $this->locale] );
 			}
 			// Apply digit transforms
 			$digits = [ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' ];
 			$letters = array_diff( $letters, $digits );
 			foreach ( $digits as $digit ) {
-				$letters[] = $this->digitTransformLanguage->formatNum( $digit, true );
+				$letters[] = $this->digitTransformLanguage->formatNumNoSeparators( $digit );
 			}
+		} elseif ( $this->locale === 'root' ) {
+			$letters = require __DIR__ . "/data/first-letters-root.php";
 		} else {
-			$letters = wfGetPrecompiledData( "first-letters-{$this->locale}.ser" );
-			if ( $letters === false ) {
-				throw new MWException( "MediaWiki does not support ICU locale " .
-					"\"{$this->locale}\"" );
-			}
+			throw new RuntimeException( "MediaWiki does not support ICU locale " .
+				"\"{$this->locale}\"" );
 		}
 
 		/* Sort the letters.
@@ -387,12 +395,10 @@ class IcuCollation extends Collation {
 				// Primary collision (two characters with the same sort position).
 				// Keep whichever one sorts first in the main collator.
 				$comp = $this->mainCollator->compare( $letter, $letterMap[$key] );
-				wfDebug( "Primary collision '$letter' '{$letterMap[$key]}' (comparison: $comp)\n" );
+				wfDebug( "Primary collision '$letter' '{$letterMap[$key]}' (comparison: $comp)" );
 				// If that also has a collision, use codepoint as a tiebreaker.
 				if ( $comp === 0 ) {
-					// TODO Use <=> operator when PHP 7 is allowed.
-					$comp = UtfNormal\Utils::utf8ToCodepoint( $letter ) -
-						UtfNormal\Utils::utf8ToCodepoint( $letterMap[$key] );
+					$comp = mb_ord( $letter ) <=> mb_ord( $letterMap[$key] );
 				}
 				if ( $comp < 0 ) {
 					$letterMap[$key] = $letter;
@@ -403,108 +409,67 @@ class IcuCollation extends Collation {
 		}
 		ksort( $letterMap, SORT_STRING );
 
-		/* Remove duplicate prefixes. Basically if something has a sortkey
+		/* Remove duplicate prefixes. Basically, if something has a sortkey
 		 * which is a prefix of some other sortkey, then it is an
 		 * expansion and probably should not be considered a section
 		 * header.
 		 *
-		 * For example 'þ' is sometimes sorted as if it is the letters
+		 * For example, 'þ' is sometimes sorted as if it is the letters
 		 * 'th'. Other times it is its own primary element. Another
-		 * example is '₨'. Sometimes its a currency symbol. Sometimes it
+		 * example is '₨'. Sometimes it's a currency symbol. Sometimes it
 		 * is an 'R' followed by an 's'.
 		 *
-		 * Additionally an expanded element should always sort directly
-		 * after its first element due to they way sortkeys work.
+		 * Additionally, an expanded element should always sort directly
+		 * after its first element due to the way sortkeys work.
 		 *
-		 * UCA sortkey elements are of variable length but no collation
+		 * UCA sortkey elements are of variable length, but no collation
 		 * element should be a prefix of some other element, so I think
 		 * this is safe. See:
-		 * - https://ssl.icu-project.org/repos/icu/icuhtml/trunk/design/collation/ICU_collation_design.htm
-		 * - http://site.icu-project.org/design/collation/uca-weight-allocation
+		 * - https://unicode-org.github.io/icu-docs/design/collation/ICU_collation_design.htm
+		 * - https://icu.unicode.org/design/collation/uca-weight-allocation
 		 *
 		 * Additionally, there is something called primary compression to
 		 * worry about. Basically, if you have two primary elements that
-		 * are more than one byte and both start with the same byte then
-		 * the first byte is dropped on the second primary. Additionally
+		 * are more than one byte and both start with the same byte, then
+		 * the first byte is dropped on the second primary. Additionally,
 		 * either \x03 or \xFF may be added to mean that the next primary
 		 * does not start with the first byte of the first primary.
 		 *
 		 * This shouldn't matter much, as the first primary is not
 		 * changed, and that is what we are comparing against.
 		 *
-		 * tl;dr: This makes some assumptions about how icu implements
+		 * tl;dr: This makes some assumptions about how ICU implements
 		 * collations. It seems incredibly unlikely these assumptions
 		 * will change, but nonetheless they are assumptions.
 		 */
 
-		$prev = false;
+		$prev = '';
 		$duplicatePrefixes = [];
 		foreach ( $letterMap as $key => $value ) {
-			// Remove terminator byte. Otherwise the prefix
-			// comparison will get hung up on that.
-			$trimmedKey = rtrim( $key, "\0" );
-			if ( $prev === false || $prev === '' ) {
-				$prev = $trimmedKey;
-				// We don't yet have a collation element
-				// to compare against, so continue.
-				continue;
-			}
-
 			// Due to the fact the array is sorted, we only have
 			// to compare with the element directly previous
 			// to the current element (skipping expansions).
 			// An element "X" will always sort directly
 			// before "XZ" (Unless we have "XY", but we
 			// do not update $prev in that case).
-			if ( substr( $trimmedKey, 0, strlen( $prev ) ) === $prev ) {
+			if ( $prev !== '' && str_starts_with( $key, $prev ) ) {
 				$duplicatePrefixes[] = $key;
 				// If this is an expansion, we don't want to
 				// compare the next element to this element,
 				// but to what is currently $prev
 				continue;
 			}
-			$prev = $trimmedKey;
+			$prev = $key;
 		}
 		foreach ( $duplicatePrefixes as $badKey ) {
-			wfDebug( "Removing '{$letterMap[$badKey]}' from first letters.\n" );
+			wfDebug( "Removing '{$letterMap[$badKey]}' from first letters." );
 			unset( $letterMap[$badKey] );
 			// This code assumes that unsetting does not change sort order.
 		}
-		$data = [
+		return [
 			'chars' => array_values( $letterMap ),
 			'keys' => array_keys( $letterMap ),
 		];
-
-		// Reduce memory usage before caching
-		unset( $letterMap );
-
-		return $data;
-	}
-
-	/**
-	 * @param string $index
-	 * @return string
-	 * @since 1.16.3
-	 */
-	public function getLetterByIndex( $index ) {
-		return $this->getFirstLetterData()['chars'][$index];
-	}
-
-	/**
-	 * @param string $index
-	 * @return string
-	 * @since 1.16.3
-	 */
-	public function getSortKeyByLetterIndex( $index ) {
-		return $this->getFirstLetterData()['keys'][$index];
-	}
-
-	/**
-	 * @return string
-	 * @since 1.16.3
-	 */
-	public function getFirstLetterCount() {
-		return count( $this->getFirstLetterData()['chars'] );
 	}
 
 	/**
@@ -514,75 +479,11 @@ class IcuCollation extends Collation {
 	 * @since 1.16.3
 	 */
 	public static function isCjk( $codepoint ) {
-		foreach ( self::$cjkBlocks as $block ) {
+		foreach ( self::CJK_BLOCKS as $block ) {
 			if ( $codepoint >= $block[0] && $codepoint <= $block[1] ) {
 				return true;
 			}
 		}
 		return false;
 	}
-
-	/**
-	 * Return the version of ICU library used by PHP's intl extension,
-	 * or false when the extension is not installed of the version
-	 * can't be determined.
-	 *
-	 * The constant INTL_ICU_VERSION this function refers to isn't really
-	 * documented. It is available since PHP 5.3.7 (see PHP 54561
-	 * https://bugs.php.net/bug.php?id=54561). This function will return
-	 * false on older PHPs.
-	 *
-	 * TODO: Remove the backwards-compatibility as MediaWiki now requires
-	 * higher levels of PHP.
-	 *
-	 * @since 1.21
-	 * @return string|bool
-	 */
-	static function getICUVersion() {
-		return defined( 'INTL_ICU_VERSION' ) ? INTL_ICU_VERSION : false;
-	}
-
-	/**
-	 * Return the version of Unicode appropriate for the version of ICU library
-	 * currently in use, or false when it can't be determined.
-	 *
-	 * @since 1.21
-	 * @return string|bool
-	 */
-	static function getUnicodeVersionForICU() {
-		$icuVersion = self::getICUVersion();
-		if ( !$icuVersion ) {
-			return false;
-		}
-
-		$versionPrefix = substr( $icuVersion, 0, 3 );
-		// Source: http://site.icu-project.org/download
-		$map = [
-			'59.' => '9.0',
-			'58.' => '9.0',
-			'57.' => '8.0',
-			'56.' => '8.0',
-			'55.' => '7.0',
-			'54.' => '7.0',
-			'53.' => '6.3',
-			'52.' => '6.3',
-			'51.' => '6.2',
-			'50.' => '6.2',
-			'49.' => '6.1',
-			'4.8' => '6.0',
-			'4.6' => '6.0',
-			'4.4' => '5.2',
-			'4.2' => '5.1',
-			'4.0' => '5.1',
-			'3.8' => '5.0',
-			'3.6' => '5.0',
-			'3.4' => '4.1',
-		];
-
-		if ( isset( $map[$versionPrefix] ) ) {
-			return $map[$versionPrefix];
-		} else {
-			return false;
-		}
-	}
 }
