@@ -20,10 +20,18 @@
  * @file
  */
 
+namespace MediaWiki\Config;
+
+use LogicException;
+
 /**
  * Exceptions for config failures
  *
+ * @newable
  * @since 1.23
  */
-class ConfigException extends Exception {
+class ConfigException extends LogicException {
 }
+
+/** @deprecated class alias since 1.41 */
+class_alias( ConfigException::class, 'ConfigException' );
