@@ -1,7 +1,5 @@
 <?php
 /**
- * Implements Special:Brokenredirects
- *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -18,41 +16,69 @@
  * http://www.gnu.org/copyleft/gpl.html
  *
  * @file
- * @ingroup SpecialPage
  */
 
-use Wikimedia\Rdbms\IResultWrapper;
+namespace MediaWiki\Specials;
+
+use MediaWiki\Cache\LinkBatchFactory;
+use MediaWiki\Content\IContentHandlerFactory;
+use MediaWiki\Page\RedirectLookup;
+use MediaWiki\SpecialPage\QueryPage;
+use MediaWiki\Title\Title;
+use Skin;
+use Wikimedia\Rdbms\IConnectionProvider;
 use Wikimedia\Rdbms\IDatabase;
+use Wikimedia\Rdbms\IResultWrapper;
 
 /**
- * A special page listing redirects to non existent page. Those should be
- * fixed to point to an existing page.
+ * List of redirects to non-existent pages.
+ *
+ * Editors are encouraged to fix these by editing them to redirect to
+ * an existing page instead.
  *
  * @ingroup SpecialPage
  */
-class BrokenRedirectsPage extends QueryPage {
-	function __construct( $name = 'BrokenRedirects' ) {
-		parent::__construct( $name );
+class SpecialBrokenRedirects extends QueryPage {
+
+	private IContentHandlerFactory $contentHandlerFactory;
+	private RedirectLookup $redirectLookup;
+
+	/**
+	 * @param IContentHandlerFactory $contentHandlerFactory
+	 * @param IConnectionProvider $dbProvider
+	 * @param LinkBatchFactory $linkBatchFactory
+	 */
+	public function __construct(
+		IContentHandlerFactory $contentHandlerFactory,
+		IConnectionProvider $dbProvider,
+		LinkBatchFactory $linkBatchFactory,
+		RedirectLookup $redirectLookup
+	) {
+		parent::__construct( 'BrokenRedirects' );
+		$this->contentHandlerFactory = $contentHandlerFactory;
+		$this->setDatabaseProvider( $dbProvider );
+		$this->setLinkBatchFactory( $linkBatchFactory );
+		$this->redirectLookup = $redirectLookup;
 	}
 
 	public function isExpensive() {
 		return true;
 	}
 
-	function isSyndicated() {
+	public function isSyndicated() {
 		return false;
 	}
 
-	function sortDescending() {
+	protected function sortDescending() {
 		return false;
 	}
 
-	function getPageHeader() {
+	protected function getPageHeader() {
 		return $this->msg( 'brokenredirectstext' )->parseAsBlock();
 	}
 
 	public function getQueryInfo() {
-		$dbr = wfGetDB( DB_REPLICA );
+		$dbr = $this->getDatabaseProvider()->getReplicaDatabase();
 
 		return [
 			'tables' => [
@@ -68,12 +94,11 @@ class BrokenRedirectsPage extends QueryPage {
 				'rd_fragment',
 			],
 			'conds' => [
-				// Exclude pages that don't exist locally as wiki pages,
-				// but aren't "broken" either.
-				// Special pages and interwiki links
-				'rd_namespace >= 0',
-				'rd_interwiki IS NULL OR rd_interwiki = ' . $dbr->addQuotes( '' ),
-				'p2.page_namespace IS NULL',
+				// Exclude pages that don't exist locally as wiki pages, but aren't "broken" either: special
+				// pages and interwiki links.
+				$dbr->expr( 'rd_namespace', '>=', 0 ),
+				'rd_interwiki' => '',
+				'p2.page_namespace' => null,
 			],
 			'join_conds' => [
 				'p1' => [ 'JOIN', [
@@ -90,31 +115,32 @@ class BrokenRedirectsPage extends QueryPage {
 	/**
 	 * @return array
 	 */
-	function getOrderFields() {
+	protected function getOrderFields() {
 		return [ 'rd_namespace', 'rd_title', 'rd_from' ];
 	}
 
 	/**
 	 * @param Skin $skin
-	 * @param object $result Result row
+	 * @param \stdClass $result Result row
 	 * @return string
 	 */
-	function formatResult( $skin, $result ) {
+	public function formatResult( $skin, $result ) {
 		$fromObj = Title::makeTitle( $result->namespace, $result->title );
 		if ( isset( $result->rd_title ) ) {
-			$toObj = Title::makeTitle( $result->rd_namespace, $result->rd_title, $result->rd_fragment );
+			$toObj = Title::makeTitle(
+				$result->rd_namespace,
+				$result->rd_title,
+				$result->rd_fragment
+			);
 		} else {
-			$blinks = $fromObj->getBrokenLinksFrom(); # TODO: check for redirect, not for links
-			if ( $blinks ) {
-				$toObj = $blinks[0];
-			} else {
-				$toObj = false;
-			}
+			$toObj = Title::castFromLinkTarget(
+				$this->redirectLookup->getRedirectTarget( $fromObj )
+			);
 		}
 
 		$linkRenderer = $this->getLinkRenderer();
-		// $toObj may very easily be false if the $result list is cached
-		if ( !is_object( $toObj ) ) {
+
+		if ( !is_object( $toObj ) || $toObj->exists() ) {
 			return '<del>' . $linkRenderer->makeLink( $fromObj ) . '</del>';
 		}
 
@@ -128,9 +154,10 @@ class BrokenRedirectsPage extends QueryPage {
 		// if the page is editable, add an edit link
 		if (
 			// check user permissions
-			$this->getUser()->isAllowed( 'edit' ) &&
+			$this->getAuthority()->isAllowed( 'edit' ) &&
 			// check, if the content model is editable through action=edit
-			ContentHandler::getForTitle( $fromObj )->supportsDirectEditing()
+			$this->contentHandlerFactory->getContentHandler( $fromObj->getContentModel() )
+				->supportsDirectEditing()
 		) {
 			$links[] = $linkRenderer->makeKnownLink(
 				$fromObj,
@@ -144,12 +171,17 @@ class BrokenRedirectsPage extends QueryPage {
 
 		$out = $from . $this->msg( 'word-separator' )->escaped();
 
-		if ( $this->getUser()->isAllowed( 'delete' ) ) {
+		if ( $this->getAuthority()->isAllowed( 'delete' ) ) {
 			$links[] = $linkRenderer->makeKnownLink(
 				$fromObj,
 				$this->msg( 'brokenredirects-delete' )->text(),
 				[],
-				[ 'action' => 'delete' ]
+				[
+					'action' => 'delete',
+					'wpReason' => $this->msg( 'brokenredirects-delete-reason' )
+						->inContentLanguage()
+						->text()
+				]
 			);
 		}
 
@@ -162,13 +194,18 @@ class BrokenRedirectsPage extends QueryPage {
 		return $out;
 	}
 
+	public function execute( $par ) {
+		$this->addHelpLink( 'Help:Redirects' );
+		parent::execute( $par );
+	}
+
 	/**
 	 * Cache page content model for performance
 	 *
 	 * @param IDatabase $db
 	 * @param IResultWrapper $res
 	 */
-	function preprocessResults( $db, $res ) {
+	public function preprocessResults( $db, $res ) {
 		$this->executeLBFromResultWrapper( $res );
 	}
 
@@ -176,3 +213,6 @@ class BrokenRedirectsPage extends QueryPage {
 		return 'maintenance';
 	}
 }
+
+/** @deprecated class alias since 1.41 */
+class_alias( SpecialBrokenRedirects::class, 'SpecialBrokenRedirects' );
