@@ -20,6 +20,8 @@
  * @file
  */
 
+use MediaWiki\Status\Status;
+
 /**
  * A class for reading ZIP file directories, for the purposes of upload
  * verification.
@@ -36,7 +38,7 @@ class ZipDirectoryReader {
 	 * behavior.
 	 *
 	 * @param string $fileName The archive file name
-	 * @param array $callback The callback function. It will be called for each file
+	 * @param callable $callback The callback function. It will be called for each file
 	 *   with a single associative array each time, with members:
 	 *
 	 *      - name: The file name. Directories conventionally have a trailing
@@ -86,54 +88,68 @@ class ZipDirectoryReader {
 	 * function should be discarded.
 	 */
 	public static function read( $fileName, $callback, $options = [] ) {
-		$zdr = new self( $fileName, $callback, $options );
-
+		$file = fopen( $fileName, 'r' );
+		$zdr = new self( $file, $callback, $options );
 		return $zdr->execute();
 	}
 
-	/** The file name */
-	protected $fileName;
+	/**
+	 * Read an opened file handle presumed to be a ZIP and call a function for
+	 * each file discovered in it.
+	 *
+	 * @see ZipDirectoryReader::read
+	 *
+	 * @param resource $file A seekable stream containing the archive
+	 * @param callable $callback
+	 * @param array $options
+	 * @return Status
+	 */
+	public static function readHandle( $file, $callback, $options = [] ) {
+		$zdr = new self( $file, $callback, $options );
+		return $zdr->execute();
+	}
 
-	/** The opened file resource */
+	/** @var resource The opened file resource */
 	protected $file;
 
-	/** The cached length of the file, or null if it has not been loaded yet. */
+	/** @var int|null The cached length of the file, or null if it has not been loaded yet. */
 	protected $fileLength;
 
-	/** A segmented cache of the file contents */
+	/** @var string[] A segmented cache of the file contents */
 	protected $buffer;
 
-	/** The file data callback */
+	/** @var callable The file data callback */
 	protected $callback;
 
-	/** The ZIP64 mode */
+	/** @var bool The ZIP64 mode */
 	protected $zip64 = false;
 
-	/** Stored headers */
-	protected $eocdr, $eocdr64, $eocdr64Locator;
-
-	protected $data;
+	/** @var array Stored headers */
+	protected $eocdr;
+	/** @var array Stored headers */
+	protected $eocdr64;
+	/** @var array Stored headers */
+	protected $eocdr64Locator;
 
 	/** The "extra field" ID for ZIP64 central directory entries */
-	const ZIP64_EXTRA_HEADER = 0x0001;
+	private const ZIP64_EXTRA_HEADER = 0x0001;
 
 	/** The segment size for the file contents cache */
-	const SEGSIZE = 16384;
+	private const SEGSIZE = 16384;
 
 	/** The index of the "general field" bit for UTF-8 file names */
-	const GENERAL_UTF8 = 11;
+	private const GENERAL_UTF8 = 11;
 
 	/** The index of the "general field" bit for central directory encryption */
-	const GENERAL_CD_ENCRYPTED = 13;
+	private const GENERAL_CD_ENCRYPTED = 13;
 
 	/**
-	 * Private constructor
-	 * @param string $fileName
+	 * @param resource $file
 	 * @param callable $callback
 	 * @param array $options
 	 */
-	protected function __construct( $fileName, $callback, $options ) {
-		$this->fileName = $fileName;
+	protected function __construct( $file, $callback, $options ) {
+		$this->file = $file;
 		$this->callback = $callback;
 
 		if ( isset( $options['zip64'] ) ) {
@@ -146,9 +162,7 @@ class ZipDirectoryReader {
 	 *
 	 * @return Status
 	 */
-	function execute() {
-		$this->file = fopen( $this->fileName, 'r' );
-		$this->data = [];
+	private function execute() {
 		if ( !$this->file ) {
 			return Status::newFatal( 'zip-file-open-error' );
 		}
@@ -157,7 +171,7 @@ class ZipDirectoryReader {
 		try {
 			$this->readEndOfCentralDirectoryRecord();
 			if ( $this->zip64 ) {
-				list( $offset, $size ) = $this->findZip64CentralDirectory();
+				[ $offset, $size ] = $this->findZip64CentralDirectory();
 				$this->readCentralDirectory( $offset, $size );
 			} else {
 				if ( $this->eocdr['CD size'] == 0xffffffff
@@ -169,7 +183,7 @@ class ZipDirectoryReader {
 						'opening vulnerabilities on clients using OpenJDK 7 or later.' );
 				}
 
-				list( $offset, $size ) = $this->findOldCentralDirectory();
+				[ $offset, $size ] = $this->findOldCentralDirectory();
 				$this->readCentralDirectory( $offset, $size );
 			}
 		} catch ( ZipDirectoryReaderError $e ) {
@@ -183,12 +197,13 @@ class ZipDirectoryReader {
 
 	/**
 	 * Throw an error, and log a debug message
-	 * @param mixed $code
+	 * @param string $code
 	 * @param string $debugMessage
 	 * @throws ZipDirectoryReaderError
+	 * @return never
 	 */
-	function error( $code, $debugMessage ) {
-		wfDebug( __CLASS__ . ": Fatal error: $debugMessage\n" );
+	private function error( $code, $debugMessage ) {
+		wfDebug( __CLASS__ . ": Fatal error: $debugMessage" );
 		throw new ZipDirectoryReaderError( $code );
 	}
 
@@ -197,7 +212,7 @@ class ZipDirectoryReader {
 	 * unimaginatively called the "end of central directory record" by the ZIP
 	 * spec.
 	 */
-	function readEndOfCentralDirectoryRecord() {
+	private function readEndOfCentralDirectoryRecord() {
 		$info = [
 			'signature' => 4,
 			'disk' => 2,
@@ -229,7 +244,9 @@ class ZipDirectoryReader {
 		$this->eocdr['EOCDR size'] = $structSize + $this->eocdr['file comment length'];
 
 		if ( $structSize + $this->eocdr['file comment length'] != strlen( $block ) - $sigPos ) {
-			$this->error( 'zip-bad', 'trailing bytes after the end of the file comment' );
+			// T40432: MS binary documents frequently embed ZIP files
+			$this->error( 'zip-wrong-format', 'there is a ZIP signature but it is not at ' .
+				'the end of the file. It could be an OLE file with a ZIP file embedded.' );
 		}
 		if ( $this->eocdr['disk'] !== 0
 			|| $this->eocdr['CD start disk'] !== 0
@@ -247,7 +264,7 @@ class ZipDirectoryReader {
 	 * Read the header called the "ZIP64 end of central directory locator". An
 	 * error will be raised if it does not exist.
 	 */
-	function readZip64EndOfCentralDirectoryLocator() {
+	private function readZip64EndOfCentralDirectoryLocator() {
 		$info = [
 			'signature' => [ 'string', 4 ],
 			'eocdr64 start disk' => 4,
@@ -272,7 +289,7 @@ class ZipDirectoryReader {
 	 * Read the header called the "ZIP64 end of central directory record". It
 	 * may replace the regular "end of central directory record" in ZIP64 files.
 	 */
-	function readZip64EndOfCentralDirectoryRecord() {
+	private function readZip64EndOfCentralDirectoryRecord() {
 		if ( $this->eocdr64Locator['eocdr64 start disk'] != 0
 			|| $this->eocdr64Locator['number of disks'] != 0
 		) {
@@ -310,7 +327,7 @@ class ZipDirectoryReader {
 	 *
 	 * @return array List containing offset, size and end position.
 	 */
-	function findOldCentralDirectory() {
+	private function findOldCentralDirectory() {
 		$size = $this->eocdr['CD size'];
 		$offset = $this->eocdr['CD offset'];
 		$endPos = $this->eocdr['position'];
@@ -331,7 +348,7 @@ class ZipDirectoryReader {
 	 *
 	 * @return array List containing offset, size and end position.
 	 */
-	function findZip64CentralDirectory() {
+	private function findZip64CentralDirectory() {
 		// The spec is ambiguous about the exact rules of precedence between the
 		// ZIP64 headers and the original headers. Here we follow zip_util.c
 		// from OpenJDK 7.
@@ -369,7 +386,7 @@ class ZipDirectoryReader {
 	 * @param int $offset
 	 * @param int $size
 	 */
-	function readCentralDirectory( $offset, $size ) {
+	private function readCentralDirectory( $offset, $size ) {
 		$block = $this->getBlock( $offset, $size );
 
 		$fixedInfo = [
@@ -462,7 +479,7 @@ class ZipDirectoryReader {
 	 * @param string $extraField
 	 * @return array|bool
 	 */
-	function unpackZip64Extra( $extraField ) {
+	private function unpackZip64Extra( $extraField ) {
 		$extraHeaderInfo = [
 			'id' => 2,
 			'size' => 2,
@@ -497,7 +514,7 @@ class ZipDirectoryReader {
 	 * Get the length of the file.
 	 * @return int
 	 */
-	function getFileLength() {
+	private function getFileLength() {
 		if ( $this->fileLength === null ) {
 			$stat = fstat( $this->file );
 			$this->fileLength = $stat['size'];
@@ -511,27 +528,25 @@ class ZipDirectoryReader {
 	 * in the file to satisfy the request, an exception will be thrown.
 	 *
 	 * @param int $start The byte offset of the start of the block.
-	 * @param int $length The number of bytes to return. If omitted, the remainder
+	 * @param int|null $length The number of bytes to return. If omitted, the remainder
 	 *    of the file will be returned.
 	 *
 	 * @return string
 	 */
-	function getBlock( $start, $length = null ) {
+	private function getBlock( $start, $length = null ) {
 		$fileLength = $this->getFileLength();
 		if ( $start >= $fileLength ) {
 			$this->error( 'zip-bad', "getBlock() requested position $start, " .
 				"file length is $fileLength" );
 		}
-		if ( $length === null ) {
-			$length = $fileLength - $start;
-		}
+		$length ??= $fileLength - $start;
 		$end = $start + $length;
 		if ( $end > $fileLength ) {
 			$this->error( 'zip-bad', "getBlock() requested end position $end, " .
 				"file length is $fileLength" );
 		}
-		$startSeg = floor( $start / self::SEGSIZE );
-		$endSeg = ceil( $end / self::SEGSIZE );
+		$startSeg = (int)floor( $start / self::SEGSIZE );
+		$endSeg = (int)ceil( $end / self::SEGSIZE );
 
 		$block = '';
 		for ( $segIndex = $startSeg; $segIndex <= $endSeg; $segIndex++ ) {
@@ -562,7 +577,7 @@ class ZipDirectoryReader {
 	 *
 	 * @return string
 	 */
-	function getSegment( $segIndex ) {
+	private function getSegment( $segIndex ) {
 		if ( !isset( $this->buffer[$segIndex] ) ) {
 			$bytePos = $segIndex * self::SEGSIZE;
 			if ( $bytePos >= $this->getFileLength() ) {
@@ -588,11 +603,11 @@ class ZipDirectoryReader {
 	 * @param array $struct
 	 * @return int
 	 */
-	function getStructSize( $struct ) {
+	private function getStructSize( $struct ) {
 		$size = 0;
 		foreach ( $struct as $type ) {
 			if ( is_array( $type ) ) {
-				list( , $fieldSize ) = $type;
+				[ , $fieldSize ] = $type;
 				$size += $fieldSize;
 			} else {
 				$size += $type;
@@ -618,13 +633,11 @@ class ZipDirectoryReader {
 	 *          Not null terminated.
 	 *
 	 * @param int $offset The offset into the string at which to start unpacking.
-	 *
-	 * @throws MWException
 	 * @return array Unpacked associative array. Note that large integers in the input
 	 *    may be represented as floating point numbers in the return value, so
 	 *    the use of weak comparison is advised.
 	 */
-	function unpack( $string, $struct, $offset = 0 ) {
+	private function unpack( $string, $struct, $offset = 0 ) {
 		$size = $this->getStructSize( $struct );
 		if ( $offset + $size > strlen( $string ) ) {
 			$this->error( 'zip-bad', 'unpack() would run past the end of the supplied string' );
@@ -634,14 +647,14 @@ class ZipDirectoryReader {
 		$pos = $offset;
 		foreach ( $struct as $key => $type ) {
 			if ( is_array( $type ) ) {
-				list( $typeName, $fieldSize ) = $type;
+				[ $typeName, $fieldSize ] = $type;
 				switch ( $typeName ) {
 					case 'string':
 						$data[$key] = substr( $string, $pos, $fieldSize );
 						$pos += $fieldSize;
 						break;
 					default:
-						throw new MWException( __METHOD__ . ": invalid type \"$typeName\"" );
+						throw new UnexpectedValueException( __METHOD__ . ": invalid type \"$typeName\"" );
 				}
 			} else {
 				// Unsigned little-endian integer
@@ -656,7 +669,7 @@ class ZipDirectoryReader {
 				}
 
 				// Throw an exception if there was loss of precision
-				if ( $value > pow( 2, 52 ) ) {
+				if ( $value > 2 ** 52 ) {
 					$this->error( 'zip-unsupported', 'number too large to be stored in a double. ' .
 						'This could happen if we tried to unpack a 64-bit structure ' .
 						'at an invalid location.' );
@@ -677,41 +690,7 @@ class ZipDirectoryReader {
 	 * @param int $bitIndex The index of the bit, where 0 is the LSB.
 	 * @return bool
 	 */
-	function testBit( $value, $bitIndex ) {
+	private function testBit( $value, $bitIndex ) {
 		return (bool)( ( $value >> $bitIndex ) & 1 );
 	}
-
-	/**
-	 * Debugging helper function which dumps a string in hexdump -C format.
-	 * @param string $s
-	 */
-	function hexDump( $s ) {
-		$n = strlen( $s );
-		for ( $i = 0; $i < $n; $i += 16 ) {
-			printf( "%08X ", $i );
-			for ( $j = 0; $j < 16; $j++ ) {
-				print " ";
-				if ( $j == 8 ) {
-					print " ";
-				}
-				if ( $i + $j >= $n ) {
-					print "  ";
-				} else {
-					printf( "%02X", ord( $s[$i + $j] ) );
-				}
-			}
-
-			print "  |";
-			for ( $j = 0; $j < 16; $j++ ) {
-				if ( $i + $j >= $n ) {
-					print " ";
-				} elseif ( ctype_print( $s[$i + $j] ) ) {
-					print $s[$i + $j];
-				} else {
-					print '.';
-				}
-			}
-			print "|\n";
-		}
-	}
 }
