@@ -1,16 +1,20 @@
-<?php
+<?php // For broken web servers: ><pre>
+
+// If you are reading this in your web browser, your server is probably
+// not configured correctly to run PHP applications!
+//
+// See the README, INSTALL, and UPGRADE files for basic setup instructions
+// and pointers to the online documentation.
+//
+// https://www.mediawiki.org/wiki/Special:MyLanguage/MediaWiki
+//
+// -------------------------------------------------
+
 /**
- * This is the main web entry point for MediaWiki.
+ * The.php entry point for web browser navigations, usually routed to
+ * an Action or SpecialPage subclass.
  *
- * If you are reading this in your web browser, your server is probably
- * not configured correctly to run PHP applications!
- *
- * See the README, INSTALL, and UPGRADE files for basic setup instructions
- * and pointers to the online documentation.
- *
- * https://www.mediawiki.org/wiki/Special:MyLanguage/MediaWiki
- *
- * ----------
+ * @see MediaWiki\Actions\ActionEntryPoint The implementation.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,13 +34,25 @@
  * @file
  */
 
+use MediaWiki\Actions\ActionEntryPoint;
+use MediaWiki\Context\RequestContext;
+use MediaWiki\EntryPointEnvironment;
+use MediaWiki\MediaWikiServices;
+
+define( 'MW_ENTRY_POINT', 'index' );
+
 // Bail on old versions of PHP, or if composer has not been run yet to install
 // dependencies. Using dirname( __FILE__ ) here because __DIR__ is PHP5.3+.
 // phpcs:ignore MediaWiki.Usage.DirUsage.FunctionFound
 require_once dirname( __FILE__ ) . '/includes/PHPVersionCheck.php';
-wfEntryPointCheck( 'index.php' );
+wfEntryPointCheck( 'html', dirname( $_SERVER['SCRIPT_NAME'] ) );
 
 require __DIR__ . '/includes/WebStart.php';
 
-$mediaWiki = new MediaWiki();
-$mediaWiki->run();
+// Create the entry point object and call run() to handle the request.
+( new ActionEntryPoint(
+	RequestContext::getMain(),
+	new EntryPointEnvironment(),
+	// TODO: Maybe create a light-weight services container here instead.
+	MediaWikiServices::getInstance()
+) )->run();
