@@ -17,16 +17,24 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  * http://www.gnu.org/copyleft/gpl.html
  *
+ * @file
  * @ingroup Maintenance
  * @author Rob Church <robchur@gmail.com>
  */
 
-require_once( dirname( __FILE__ ) . '/Maintenance.php' );
+// @codeCoverageIgnoreStart
+require_once __DIR__ . '/Maintenance.php';
+// @codeCoverageIgnoreEnd
 
+/**
+ * Maintenance script that purges old text records from the database.
+ *
+ * @ingroup Maintenance
+ */
 class PurgeOldText extends Maintenance {
 	public function __construct() {
 		parent::__construct();
-		$this->mDescription = "Purge old text records from the database";
+		$this->addDescription( 'Purge old text records from the database' );
 		$this->addOption( 'purge', 'Performs the deletion' );
 	}
 
@@ -35,5 +43,7 @@ class PurgeOldText extends Maintenance {
 	}
 }
 
-$maintClass = "PurgeOldText";
-require_once( RUN_MAINTENANCE_IF_MAIN );
+// @codeCoverageIgnoreStart
+$maintClass = PurgeOldText::class;
+require_once RUN_MAINTENANCE_IF_MAIN;
+// @codeCoverageIgnoreEnd
